==> pam-0.3.tar.gz: Linux-PAM-0.3/libpam/pam_misc.c <==
/* pam_misc.c -- This is random stuff */

/* $Id: pam_misc.c,v 1.2 1996/03/16 21:55:13 morgan Exp $
 *
 * $Log: pam_misc.c,v $
 * Revision 1.2  1996/03/16 21:55:13  morgan
 * changed pam_mkargv to _pam_mkargv
 *
 */

#include <stdarg.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <syslog.h>
#include <ctype.h>

#include "pam_private.h"

int strCMP(const char *s, const char *t)   /* caseless string comparison */
{
    int cf;

    do {
	cf = tolower(*s) - tolower(*t);
	++t;
    } while (*s++ && !cf);

    return cf;
}

char *strdup(const char *s)
{
    char *p = malloc(strlen(s) + 1);
    if (p)
	strcpy(p, s);
    else {
	_pam_log_error("strdup: failed to get memory");
    }
    return p;
}

/* Generate argv, argc from s */
/* caller must free(argv)     */

int _pam_mkargv(char *s, char ***argv, int *argc)
{
    int l;
    int argvlen = 0;
    char *sbuf, *sbuf_start;
    char **our_argv = NULL;
    char **argvbuf;
    char *argvbufp;

    *argc = 0;

    l = strlen(s);
    if (l) {
	if ((sbuf = sbuf_start = strdup(s)) == NULL) {
	    _pam_log_error("pam_mkargv: null returned by strdup");
	} else {
	    /* Overkill on the malloc, but not large */
	    argvlen = l * ((sizeof(char)) + sizeof(char *));
	    if ((our_argv = argvbuf = malloc(argvlen)) == NULL) {
		_pam_log_error("pam_mkargv: null returned by malloc");
	    } else {
		argvbufp = (char *) argvbuf + (l * sizeof(char *));
		while ((sbuf = strtok(sbuf, " \n\t"))) {
		    strcpy(argvbufp, sbuf);
		    *argvbuf = argvbufp;
		    argvbufp += strlen(argvbufp) + 1;
	    
		    (*argc)++;
		    argvbuf++;
		    sbuf = NULL;
		}
		DROP(sbuf_start);
	    }
	}
    }
    
    *argv = our_argv;
    return(argvlen);
}

void _pam_log_error(const char *format, ...)
{
    char buf[1024];
    va_list args;

    va_start(args, format);
    vsprintf(buf, format, args);

#ifdef DEBUG
#define ADDPERROR |LOG_PERROR
#else
#define ADDPERROR
#endif

    openlog("pam", LOG_CONS|LOG_PID ADDPERROR, LOG_AUTH);
    syslog(LOG_CRIT, buf);
    closelog();
}

#ifdef DEBUG
void _pam_debug(const char *format, ...)
{
    va_list args;

    va_start(args, format);
    vfprintf(stderr, format, args);
    fprintf(stderr, "\n");
}
#endif

==> pam-0.21.tar.gz: Linux-PAM-0.21/libpam/pam_misc.c <==
/* pam_misc.c -- This is random stuff */

#include <stdarg.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <syslog.h>
#include <ctype.h>

#include "pam_private.h"

int strCMP(const char *s, const char *t)   /* caseless string comparison */
{
    int cf;

    do {
	cf = tolower(*s) - tolower(*t);
	++t;
    } while (*s++ && !cf);

    return cf;
}

char *strdup(const char *s)
{
    char *p = malloc(strlen(s) + 1);
    if (p)
	strcpy(p, s);
    else {
	_pam_log_error("strdup: failed to get memory");
    }
    return p;
}

/* Generate argv, argc from s */
/* caller must free(argv)     */

int pam_mkargv(char *s, char ***argv, int *argc)
{
    int l;
    int argvlen = 0;
    char *sbuf, *sbuf_start;
    char **our_argv = NULL;
    char **argvbuf;
    char *argvbufp;

    *argc = 0;

    l = strlen(s);
    if (l) {
	if ((sbuf = sbuf_start = strdup(s)) == NULL) {
	    _pam_log_error("pam_mkargv: null returned by strdup");
	} else {
	    /* Overkill on the malloc, but not large */
	    argvlen = l * ((sizeof(char)) + sizeof(char *));
	    if ((our_argv = argvbuf = malloc(argvlen)) == NULL) {
		_pam_log_error("pam_mkargv: null returned by malloc");
	    } else {
		argvbufp = (char *) argvbuf + (l * sizeof(char *));
		while ((sbuf = strtok(sbuf, " \n\t"))) {
		    strcpy(argvbufp, sbuf);
		    *argvbuf = argvbufp;
		    argvbufp += strlen(argvbufp) + 1;
	    
		    (*argc)++;
		    argvbuf++;
		    sbuf = NULL;
		}
		DROP(sbuf_start);
	    }
	}
    }
    
    *argv = our_argv;
    return(argvlen);
}

void _pam_log_error(const char *format, ...)
{
    char buf[1024];
    va_list args;

    va_start(args, format);
    vsprintf(buf, format, args);

#ifdef DEBUG
#define ADDPERROR |LOG_PERROR
#else
#define ADDPERROR
#endif

    openlog("pam", LOG_CONS|LOG_PID ADDPERROR, LOG_AUTH);
    syslog(LOG_CRIT, buf);
    closelog();
}

#ifdef DEBUG
void _pam_debug(const char *format, ...)
{
    va_list args;

    va_start(args, format);
    vfprintf(stderr, format, args);
    fprintf(stderr, "\n");
}
#endif
